function stats = QPbatch(set,idxset)
% QPBATCH  Run set of QP problems
%
%     QPBATCH(SET) run complete SET of test problems
%
%     SET - 'A'    100x200 Gaussian matrix

solvers = {'bgs','spg','spg-base','pg','smooth'};
solverLambda = 1; % Default solver for lambda formulation
solverTau    = 2; % Default solver for tau formulation
solverXnorm  = 2; % Solver to compare solution to

[pathstr,name,ext,versn] = fileparts(mfilename('fullpath'));

switch lower(set)


   % -----------------------------------------------------
   % A - 100x200 Gaussian matrix (BGS was ran first)
   % -----------------------------------------------------
   case {'a'}
      instances = 10;
      generate  = QPproblem('a');

      opts = struct();
      opts.verbosity = 0;
      opts.maxiter   = 1e4;
      opts.maxfunc   = 1e5;
      opts.bgstol    = 1e-4;
      opts.epsl1tol  = 1e-8;
      opts.epsilon   = 5e-10;

   % -----------------------------------------------------
   % B - 300x2000 Gaussian matrix (BGS was ran first)
   % -----------------------------------------------------
   case {'b'}
      instances = 100;
      generate  = QPproblem('b');

      opts = struct();
      opts.verbosity = 0;
      opts.maxiter   = 1e4;
      opts.maxfunc   = 1e5;
      opts.bgstol    = 1e-4;
      opts.epsl1tol  = 1e-8;
      opts.epsilon   = 5e-10;

   % -----------------------------------------------------
   % C - 300x2048 Restricted DCT (BGS was ran first)
   % -----------------------------------------------------
   case {'c'}
      instances = 20;
      generate  = QPproblem('c');

      opts = struct();
      opts.verbosity = 0;
      opts.maxiter   = 1e4;
      opts.maxfunc   = 1e5;
      opts.bgstol    = 1e-4;
      opts.epsl1tol  = 1e-8;
      opts.epsilon   = 5e-10;

   % -----------------------------------------------------
   % D - 1200x8192 Restricted DCT (SPG first)
   % -----------------------------------------------------
   case {'d'}
      instances = 50;
      generate  = QPproblem('d');

      opts = struct();
      opts.verbosity = 0;
      opts.maxiter   = 1e4;
      opts.maxfunc   = 2000;
      opts.bgstol    = 1e-4;
      opts.epsl1tol  = 1e-8;
      opts.epsilon   = 5e-10;

   % -----------------------------------------------------
   % E - 4800x32768 Restricted DCT (SPG first)
   % -----------------------------------------------------
   case {'e'}
      instances = 50;
      generate  = QPproblem('e');

      opts = struct();
      opts.verbosity = 0;
      opts.maxiter   = 1e4;
      opts.maxfunc   = 5000;
      opts.bgstol    = 1e-4;
      opts.epsl1tol  = 1e-8;
      opts.epsilon   = 5e-10;

   % -----------------------------------------------------
   % F - 9600x65536 Restricted DCT (SPG first)
   % -----------------------------------------------------
   case {'f'}
      instances = 50;
      generate  = QPproblem('f');

      opts = struct();
      opts.verbosity = 0;
      opts.maxiter   = 1e4;
      opts.maxfunc   = 5000;
      opts.bgstol    = 1e-4;
      opts.epsl1tol  = 1e-8;
      opts.epsilon   = 5e-10;
      
   % -----------------------------------------------------
   % Unknown problem set - error
   % -----------------------------------------------------
   otherwise
      error('Unknown problem set');
end

if nargin < 2, idxset = 1:instances; end;

stats   = cell(length(solvers),instances);

for i=idxset

   % Create filename
   filename = sprintf('%s%sdata%sQP_%s_%03d.mat', ...
                      pathstr,filesep,filesep,upper(set),i);

   % Check if solution already exists
   if exist(filename)
      fprintf('%s Instance %4d: Loading %s . . .\n',upper(set),i,filename);
      data = load(filename,'info');
      info = data.info;
   else
      fprintf('%s Instance %4d: Solving . . .\n',upper(set),i);

      % Initialize
      tau  = 0;
      x    = cell(length(solvers),1); 
      info = cell(length(solvers),1);
   

      % Generate problem
      [A,b,x0,groups,weights,lambda,tau] = generate(i);

      % Determine which solver to run first
      if isnan(tau)
         % Run first solver (must have lambda formulation)
         [x{solverLambda},info{solverLambda}] = ...
             QPrun(A,b,groups,weights,lambda,tau,solvers{solverLambda},opts);
         fprintf('         Done running %s\n', solvers{solverLambda});
         
         % Compute value of tau and compute remaining solutions
         tau = sum(weights.*sqrt(accumarray(groups, (x{solverLambda}).^2)));
         
         % Set remaining solvers
         jIdx = setdiff(1:length(solvers),solverLambda);
      else
         % Run first solver (must have tau formulation)
        [x{solverTau},info{solverTau}] = ...
            QPrun(A,b,groups,weights,lambda,tau,solvers{solverTau},opts);
         fprintf('         Done running %s\n', solvers{solverTau});

        % Compute value of lambda and compute remaining solutions
        if isa(A,'function_handle')
           r = b - A(x{solverTau},1);
           Atr = A(r,2);
        else          
           r = b - A*x{solverTau};
           Atr = (r'*A)';
        end
        lambda = max(sqrt(accumarray(groups,Atr.^2))./weights);
      
         % Set remaining solvers
         jIdx = setdiff(1:length(solvers),solverTau);
      end
      

      for j=jIdx
         [x{j},info{j}] = QPrun(A,b,groups,weights,lambda,tau, ...
                                solvers{j},opts);
         fprintf('         Done running %s\n', solvers{j});
      end
      
      
      % Compare the norm of the solutions
      for j=1:length(solvers)
         info{j}.normDiff = norm(x{j}-x{solverXnorm});
      end

      % Store results
      save(filename,'info');
      
        figure(1); clf; gca; hold on;
        marker = {'r.','b.','ro','bo','g.'};
        for k=1:length(solvers)
          plot(x{k},marker{k});
        end
        plot(x0,'kx'); pause(1);
   end
   
   stats(:,i) = info;
end
