function [x,stats] = QPrun(A,b,groups,weights,lambda,tau,solver,opts)

% Create objective function and projector
if isa(A,'function_handle')
   funObj = @(x) QPfunimplicit(A,b,x);
else
   funObj = @(x) QPfun(A,b,x);
end

funProjTau = GroupOneNormProject(groups,weights,1);
funProj    = @(x) funProjTau(x,tau);
funGNorm   = @(x) sum(weights.*sqrt(accumarray(groups,x.^2)));


% Start watches
tic; t0 = cputime;

switch (lower(solver))

   case {'bgs'}
      [x,info] = bgs(funObj,lambda,groups,weights,opts);
      stats.iter  = info.iter;
      stats.feval = max(info.evalHist);

   case {'spg'} % With projected backtracking
      opts.curvilinear = 1; % Projected backtracking
      opts.spectral    = 1; % Enable Barzilai-Borwein
      [x,g,info] = spg(funObj, funProj, zeros(length(groups),1), opts);
      stats.iter  = info.iter;
      stats.feval = max(info.evalHist);

   case {'spg-base'} % No projected backtracking
      opts.curvilinear = 0; % No projected backtracking
      opts.spectral    = 1; % Enable Barzilai-Borwein
      [x,g,info] = spg(funObj, funProj, zeros(length(groups),1), opts);
      stats.iter  = info.iter;
      stats.feval = max(info.evalHist);

   case {'pg'} % Projected gradient
      opts.curvilinear = 1; % Projected backtracking
      opts.spectral    = 0; % Disable Barzilai-Borwein
      opts.history     = 1; % Monotone iterations
      [x,g,info] = spg(funObj, funProj, zeros(length(groups),1), opts);
      stats.iter  = info.iter;
      stats.feval = max(info.evalHist);

   case {'smooth'}
      [x,info] = epsL1(funObj,lambda,groups,weights,opts);
      stats.iter  = info.iter;
      stats.feval = max(info.evalHist);

end

% Record all information
stats.tictoc   = toc;
stats.cputime  = cputime - t0;
stats.solver   = solver;
stats.info     = info;
stats.tau      = tau;
stats.lambda   = lambda;
stats.f        = funObj(x);
stats.xGrpNorm = funGNorm(x);
stats.fPenalty = stats.f + lambda * stats.xGrpNorm;



function [f,g] = QPfun(A,b,x)
r = A*x - b;
f = r'*r / 2;
g = (r' * A)';


function [f,g] = QPfunimplicit(A,b,x)
r = A(x,1) - b;
f = r'*r / 2;
g = A(r,2);
