function f = GroupOneNormProject(groups,weights,mode)
% GROUPONENORMPROJECT   Group projection onto one-norm ball
%
%   F = GROUPONENORMPROJECT(GROUPS,WEIGHTS,FUN) creates a function
%   handle for projection onto a (weighted) one-norm ball. GROUPS
%   can either be an array with group numbers or a cell array of
%   the format {N,GROUPSIZE}, where N is the number of groups and
%   GROUPSIZE is the number of elements per group. If GROUPSIZE is
%   negative the stride between members of the same group is taken
%   as N, otherwise it is taken to be one (i.e., the members of the
%   group are in adjacent entries of the vector). If the WEIGHTS
%   parameter is specified it should match the number of groups.
%   The MODE parameter specifies the core projection function to
%   use:
%
%   MODE:  [1]  O(n log n) time algorithm (heap)
%           2   O(n) time algorithm       (random)
%           3   O(n log n) time algorithm (sort)
%
%   The function F takes parameters X and TAU and returns the
%   projection of X onto the underlying one-norm ball of radius
%   TAU.

% Check WEIGHTS parameter
if nargin < 1 || isempty(weights)
   weights = 1;
end

% Check MODE parameter
if nargin < 2, mode = 1; end;

if mode == 1
  funI = @projectHeapC;
  funW = @projectHeapC;
elseif mode == 2
  funI = @projectRandomC;
  funW = @projectWRandom;
else
  funI = @projectSortC;
  funW = @projectSort; % MS: added this because projectHeapC was returning NaN
  % MS: NOTE: projectSort does not seem to work when one of the weights is 0
end

% Accept either a cell array of group indices or a vector with group numbers
if iscell(groups)
   % Check format
   if length(groups) < 2
      error('Parameter GROUPS should contain exactly two entries in cell-format');
   end
   
   % Get number of groups and group size
   n = groups{1};
   s = groups{2};

   % Check WEIGHTS parameter
   if ~isscalar(weights) && length(weights) ~= n
      error('Length of WEIGTHS vector must match number of groups');
   end
   
   % Create function
   if isscalar(weights)
      if s < 0
         f = @(x,tau) projectIStrideN(n,-s,funI,x,tau/weights);
      else
         f = @(x,tau) projectIStride1(n, s,funI,x,tau/weights);
      end      
   else
      idx = (weights ~= 0);
      if s < 0
         f = @(x,tau) projectWStrideN(n,-s,weights,idx,funW,x,tau);
      else
         f = @(x,tau) projectWStride1(n, s,weights,idx,funW,x,tau);
      end      
   end
else
   % Ensure proper setting of groups
   groups = round(groups);
   mn     = min(groups);
   if (mn ~= 1)
      groups = groups - mn + 1;
   end
   
   if isscalar(weights)
      f = @(x,tau) projectIGroups(groups,funI,x,tau/weights);
   else
      idx = (weights ~= 0);
      f = @(x,tau) projectWGroups(groups,weights,idx,funW,x,tau);
   end
   
end

end


function [y,val] = projectIStride1(n,s,fun,x,tau)
   v   = reshape(x,s,n);
   g0  = sqrt(sum(v.^2,1))';
   val = sum(g0);   % Group norm of the vector
   g1  = fun(g0,tau);
   g0(g0 == 0) = 1; % Avoid division by zero
   d   = spdiags(g1./g0,0,n,n);
   v   = v * d;
   y   = reshape(v,n*s,1);
end

function [y,val] = projectIStrideN(n,s,fun,x,tau)
   v   = reshape(x,n,s);
   g0  = sqrt(sum(v.^2,2));
   val = sum(g0);   % Group norm of the vector
   g1  = fun(g0,tau);
   g0(g0 == 0) = 1; % Avoid division by zero
   d   = spdiags(g1./g0,0,n,n);
   v   = d * v;
   y   = reshape(v,n*s,1);
end

function [y,val] = projectWStride1(n,s,w,idx,fun,x,tau)
   v   = reshape(x,s,n);
   g0  = sqrt(sum(v.^2,1))';
   val = sum(weights.*g0); % Group norm of the vector
   g1  = g0;
   g1(idx) = fun(g0(idx),w(idx),tau);
   g0(g0 == 0) = 1; % Avoid division by zero
   d   = spdiags(g1./g0,0,n,n);
   v   = v * d;
   y   = reshape(v,n*s,1);
end

function [y,val] = projectWStrideN(n,s,w,idx,fun,x,tau)
   v   = reshape(x,n,s);
   g0  = sqrt(sum(v.^2,2));
   val = sum(weights.*g0); % Group norm of the vector
   g1  = g0;
   g1(idx) = fun(g0(idx),w(idx),tau);
   g0(g0 == 0) = 1; % Avoid division by zero
   d   = spdiags(g1./g0,0,n,n);
   v   = d * v;
   y   = reshape(v,n*s,1);
end

function [y,val] = projectIGroups(groups,fun,x,tau)
   g0  = sqrt(accumarray(groups,x.^2)); % MS: added sqrt()
   val = sum(g0);   % Group norm of the vector
   g1  = fun(g0,tau); 
   g0(g0 == 0) = 1; % Avoid division by zero
   s   = g1./g0;
   y   = x .* s(groups); % MS: changed LHS to 
end

function [y,val] = projectWGroups(groups,weights,idx,fun,x,tau)
   g0  = sqrt(accumarray(groups,x.^2)); % MS: added sqrt()
   val = sum(weights.*g0); % Group norm of the vector
   g1  = g0;
   g1(idx) = fun(g0(idx),weights(idx),tau);
   g0(g0 == 0) = 1; % Avoid division by zero
   s   = g1./g0;
   y   = x .* s(groups);
end
